<?php

declare(strict_types=1);

namespace Drupal\entitylogic\Plugin\views\field;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;

/**
 * Provides Entitylogic Function field handler.
 *
 * @ViewsField("entitylogic_method_call")
 *
 * @DCG
 * The plugin needs to be assigned to a specific table column through
 * hook_views_data() or hook_views_data_alter().
 * Put the following code to entitylogic.views.inc file.
 * @code
 * function foo_views_data_alter(array &$data): void {
 *   $data['node']['foo_example']['field'] = [
 *     'title' => t('Example'),
 *     'help' => t('Custom example field.'),
 *     'id' => 'foo_example',
 *   ];
 * }
 * @endcode
 */
final class MethodCall extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['method_name'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);
    $form['method_name'] = [
      '#type' => 'textfield',
      '#title' => 'Method name',
      '#default_value' => $this->options['method_name'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    // NOP
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values): string|MarkupInterface {
    $value = parent::render($values);

    $wEntity = entitylogic($values->_entity);

    $method_name = $this->options['method_name'];

    if (method_exists($wEntity, $method_name)) {
      $value = call_user_func([$wEntity, $method_name]);

      return $this->sanitizeValue($value);
    }

    return '';
  }

}
